from .NotAuthException import NotAuthException
from .NotFoundException import NotFoundException
from .ExceptionBuilder import ExceptionBuilder
from .ResponseBuilder import ResponseBuilder


class ExceptionInterceptor:
    def error(self, error: Exception):
        return ResponseBuilder.failed(ExceptionBuilder.build(error))

    def notFound(self, error: NotFoundException):
        return ResponseBuilder.failedNotFound(ExceptionBuilder.build(error))

    def notAuth(self, error: NotAuthException):
        return ResponseBuilder.responseConfig(error, statusBody = {"message": "No authority in route", "status": 403})



import string

class Response:
    def __init__(self, data: object = None, message: string = None, status: int = None) -> None:
        self.data = data
        self.message = message
        self.status = status
    



from flask import jsonify, make_response
from .response import Response
from .ExceptionBuilder import ExceptionBuilder
from .ExceptionView import ExceptionView


class ResponseBuilder:
    @staticmethod
    def success(object: object) -> Response:
        if object.__class__.__name__ != 'list':
            object = object.__dict__
        
        response = Response(
            data = object,
            message = "Proccess to success in server",
            status = 200
        )
        
        return jsonify(response.__dict__)

    @staticmethod
    def voidSuccess(func, object: object = None) -> Response:
        if object == None:
            func()
        else:
            func(object)

        response = Response(
            data = None,
            message = "Proccess to success in server",
            status = 200
        )
        
        return jsonify(response.__dict__)

    @staticmethod
    def failed(exception) -> Response:
        ExceptionView.build(exception)

        response = Response(
            data = None,
            message = "An error has occurred on the server",
            status = 500
        )
        
        return make_response(jsonify(response.__dict__), 500)

    @staticmethod
    def failedNotFound(exception) -> Response:
        ExceptionView.build(exception)

        response = Response(
            data = None,
            message = "Not found data in server",
            status = 404
        )
        
        return make_response(jsonify(response.__dict__), 404)

    @staticmethod
    def responseConfig(exception = None, data: object = None, statusBody: dict = None) -> Response:
        if exception != None:
            ExceptionView.build(ExceptionBuilder.build(exception))

        response = Response(
            data = data,
            message = statusBody['message'],
            status = statusBody['status']
        )
        
        return make_response(jsonify(response.__dict__), statusBody['status'])
        


from distutils.log import error
from .exceptionBody import ExceptionBody

class ExceptionView:
    
    @staticmethod
    def build(exception: ExceptionBody):
        error("{")

        print("   Cause: " + str(exception.cause))
        print("   Context: " + str(exception.context))
        print("   Traceback: " + str(exception.traceback))
        print("   Classname: " + str(exception.clazz))

        error("}")



class ExceptionBody:
    def __init__(self, cause = None, 
                       traceback = None, 
                       context = None, 
                       clazz = None, 
                       name = None) -> None:
        self.cause = cause
        self.traceback = traceback
        self.context = context
        self.clazz = clazz



import pathlib
from setuptools import setup

HERE = pathlib.Path(__file__).parent

VERSION = '0.0.5'
PACKAGE_NAME = 'Common-OurKids' 
AUTHOR = 'Jefry Zárate Ruíz' 
AUTHOR_EMAIL = 'jefryzarateruiz@gmail.com'
URL = 'https://github.com/Jefry-bot/Common-OurKids'
DOWNLOAD_URL= URL + '/tarball/0.0.1'

LICENSE = 'MIT' 
DESCRIPTION = 'Librería para tener configuraciones y archivos comunes para la implementación y uso de servidores en flask.'
LONG_DESCRIPTION = (HERE / "README.md").read_text(encoding='utf-8') 
LONG_DESC_TYPE = "text/markdown"

INSTALL_REQUIRES = [
      'python-dotenv',
      'flask',
      'pymysql'
      ]

setup(
    name=PACKAGE_NAME,
    version=VERSION,
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    long_description_content_type=LONG_DESC_TYPE,
    author=AUTHOR,
    author_email=AUTHOR_EMAIL,
    url=URL,
    download_url=DOWNLOAD_URL,
    install_requires=INSTALL_REQUIRES,
    keywords=['testing', 'logging', 'example'],
    classifiers=[],
    license=LICENSE,
    packages= ['Common-OurKids'],
    include_package_data=True
)


from flask import Flask
from .load import load_env
load_env()

from interceptors import INTERCEPTORS
from NotFoundException import NotFoundException
from NotAuthException import NotAuthException
from ExceptionInterceptor import ExceptionInterceptor

def config(app: Flask, CONTROLLERS):
    interceptor = ExceptionInterceptor()
    
    for k in CONTROLLERS.keys():
        eval("app.register_blueprint(CONTROLLERS.get(k)." + str(k) + "_routes, url_prefix='/api')")
    for k, v in INTERCEPTORS.items():
        eval("app.register_error_handler(" + str(v) + ", interceptor." + str(k) + ")")


from os import getenv
import pymysql

from .load import load_env

class DataBase:
    def __init__(self) -> None: 
        self.connection = pymysql.connect(
            host = getenv('DB_HOST'), 
            user = getenv('DB_USER'),
            password = getenv('DB_PASSWORD'),
            port= int(getenv('DB_PORT')),
            db = getenv('DB')
        )
        self.cursor = self.connection.cursor()

    def commit(self) -> None:
        self.connection.commit()

    def close(self) -> None:
        self.connection.close()


from .exceptionBody import ExceptionBody
from .NotFoundException import NotFoundException

class ExceptionBuilder:
    
    @staticmethod
    def build(exception: BaseException) -> ExceptionBody:
        exceptionBody = ExceptionBody(
            cause = exception.__cause__,
            context = exception.__context__,
            traceback = exception.__traceback__,
            clazz = exception.__class__
        )
        
        return exceptionBody